#!/usr/bin/env python

# Raspi-sump, a sump pump monitoring system.
# Al Audet
# https://www.linuxnorth.org/raspi-sump/
#
# All configuration changes should be done in raspisump.conf
# MIT License -- https://www.linuxnorth.org/raspi-sump/license.html

import time
from raspisump import webchart


def main():
    '''Pass variables to webchart.py'''
    year = time.strftime('%Y')
    month = time.strftime('%m')
    today = time.strftime('%Y%m%d')
    homedir = '/home/pi/raspi-sump/'
    webchart.create_folders(year, month, homedir)
    webchart.create_chart(homedir)
    webchart.copy_chart(year, month, today, homedir)

if __name__ == '__main__':
    main()



"""Send an email on command to test."""

# Raspi-sump, a sump pump monitoring system.
# Al Audet
# https://www.linuxnorth.org/raspi-sump/
#
# All configuration changes should be done in raspisump.conf
# MIT License -- https://www.linuxnorth.org/raspi-sump/license.html

import smtplib
import configparser
from raspisump import alerts


config = configparser.RawConfigParser()

config.read("/home/pi/raspi-sump/raspisump.conf")

configs = {
    "email_to": config.get("email", "email_to"),
    "email_from": config.get("email", "email_from"),
    "smtp_authentication": config.getint("email", "smtp_authentication"),
    "smtp_tls": config.getint("email", "smtp_tls"),
    "smtp_server": config.get("email", "smtp_server"),
    "username": config.get("email", "username"),
    "password": config.get("email", "password"),
}


def test_email_content():

    """Build the contents of test email body."""

    time_of_day = alerts.current_time()
    hostname = alerts.host_name()

    subject = "Subject: Raspi-Sump Email Test"
    message = "Raspi-Sump Test Email"

    return "\r\n".join(
        (
            "From: {}".format(configs["email_from"]),
            "To: {}".format(configs["email_to"]),
            "{}".format(subject),
            "",
            "{} - {} - {}.".format(hostname, time_of_day, message),
        )
    )


def test_email():
    """Send test email only."""
    recipients = configs["email_to"].split(", ")
    email_body = test_email_content()
    server = smtplib.SMTP(configs["smtp_server"])

    # Check if smtp server uses TLS
    if configs["smtp_tls"] == 1:
        server.starttls()
    else:
        pass
    # Check if smtp server uses authentication
    if configs["smtp_authentication"] == 1:
        username = configs["username"]
        password = configs["password"]
        server.login(username, password)
    else:
        pass

    server.sendmail(configs["email_from"], recipients, email_body)
    server.quit()



"""Create charts for viewing on Raspberry Pi Web Server."""

# Raspi-sump, a sump pump monitoring system.
# Al Audet
# https://www.linuxnorth.org/raspi-sump/
#
# All configuration changes should be done in raspisump.conf
# MIT License -- https://www.linuxnorth.org/raspi-sump/license.htmlimport os

import os
import subprocess
import time
from raspisump import todaychart


def create_folders(year, month, homedir):
    """Check if folders exist in charts folder and create them if they don't"""
    if not os.path.isdir("{}charts/{}/".format(homedir, year)):
        _year = "mkdir {}charts/{}".format(homedir, year)
        create_year = _year.split(" ")
        subprocess.call(create_year)

    if not os.path.isdir("{}charts/{}/{}/".format(homedir, year, month)):
        _month = "mkdir {}charts/{}/{}".format(homedir, year, month)
        create_month = _month.split(" ")
        subprocess.call(create_month)


def create_chart(homedir):
    """Create a chart of sump pit activity and save to web folder"""
    csv_file = "{}csv/waterlevel-{}.csv".format(homedir, time.strftime("%Y%m%d"))
    filename = "{}charts/today.png".format(homedir)
    todaychart.graph(csv_file, filename)


def copy_chart(year, month, today, homedir):
    """Copy today.png to year/month/day folder for web viewing"""
    copy_cmd = "cp {}charts/today.png {}charts/{}/{}/{}.png".format(
        homedir, homedir, year, month, today
    )
    copy_file = copy_cmd.split(" ")
    subprocess.call(copy_file)



'''Send Heartbeat alert to test that email notifications are working.'''

# Raspi-sump, a sump pump monitoring system.
# Al Audet
# https://www.linuxnorth.org/raspi-sump/
#
# All configuration changes should be done in raspisump.conf
# MIT License -- https://www.linuxnorth.org/raspi-sump/license.html

import os
import time
import smtplib
from datetime import datetime, timedelta
import configparser
from collections import deque
import csv
from raspisump import log, alerts


config = configparser.RawConfigParser()

config.read('/home/pi/raspi-sump/raspisump.conf')

configs = {'email_to': config.get('email', 'email_to'),
           'email_from': config.get('email', 'email_from'),
           'smtp_authentication': config.getint(
           'email', 'smtp_authentication'),
           'smtp_tls': config.getint('email', 'smtp_tls'),
           'smtp_server': config.get('email', 'smtp_server'),
           'username': config.get('email', 'username'),
           'password': config.get('email', 'password')
           }

try:
    configs['heartbeat_interval'] = config.getint('email',
                                                  'heartbeat_interval')
except configparser.NoOptionError:
    configs['heartbeat_interval'] = 10080


def get_last_alert_time():
    '''Retrieve the last alert time string from logfile'''
    heartbeat_log = '/home/pi/raspi-sump/logs/heartbeat_log'
    with open(heartbeat_log, 'rt') as f:
        last_row = deque(csv.reader(f), 1)[0]
        return last_row[0]


def heartbeat_email_content():
    '''Build the contents of email body which will be sent as an alert'''
    heartbeat_interval_time = configs['heartbeat_interval']
    current_time = time.strftime('%Y-%m-%d %H:%M:%S')
    last_alert = datetime.strptime(current_time, '%Y-%m-%d %H:%M:%S')
    future_date = last_alert + timedelta(minutes=heartbeat_interval_time + 1)

    weekday = future_date.strftime('%A')[0:3]
    month = future_date.strftime('%B')
    hour = future_date.strftime('%I')
    minute = future_date.strftime('%M')
    am_pm = future_date.strftime('%p').lower()

    time_of_day = alerts.current_time()
    hostname = alerts.host_name()
    subject = 'Subject: Raspi-Sump Heartbeat Notification'
    message = 'Raspi-Sump Email Notifications Working'

    return "\r\n".join((
        "From: {}".format(configs['email_from']),
        "To: {}".format(configs['email_to']),
        "{}".format(subject),
        "",
        "{} - {} - {}.".format(hostname, time_of_day, message),
        "Next heartbeat: {} {} {} at {}:{} {}".format(weekday,
                                                      month,
                                                      future_date.day,
                                                      hour,
                                                      minute,
                                                      am_pm),
        )
        )


def heartbeat_alerts():
    '''Send heartbeat email alert if water level greater
    than critical distance.'''
    recipients = configs['email_to'].split(', ')
    email_body = heartbeat_email_content()
    server = smtplib.SMTP(configs['smtp_server'])

    # Check if smtp server uses TLS
    if configs['smtp_tls'] == 1:
        server.starttls()
    else:
        pass
    # Check if smtp server uses authentication
    if configs['smtp_authentication'] == 1:
        username = configs['username']
        password = configs['password']
        server.login(username, password)
    else:
        pass

    server.sendmail(configs['email_from'], recipients, email_body)
    server.quit()


def determine_if_heartbeat():
    '''Determine if a heartbeat notification is required and if so, send
    the notification.'''

    heartbeat_log = '/home/pi/raspi-sump/logs/heartbeat_log'
    if not os.path.isfile(heartbeat_log):
        heartbeat_alerts()
        log.log_event("heartbeat_log", "Heartbeat Email Sent")

    else:
        heartbeat_interval_time = configs['heartbeat_interval']
        last_email_sent = get_last_alert_time()
        current_time = time.strftime('%Y-%m-%d %H:%M:%S')
        last_heartbeat_time = datetime.strptime(last_email_sent,
                                                '%Y-%m-%d %H:%M:%S')
        time_now = datetime.strptime(current_time, '%Y-%m-%d %H:%M:%S')
        delta = (time_now - last_heartbeat_time)
        minutes_passed = int((delta).total_seconds() / 60)

        if minutes_passed >= heartbeat_interval_time:
            heartbeat_alerts()
            log.log_event("heartbeat_log", "Heartbeat Email Sent")
        else:
            pass



""" Module to take a water_level reading."""

# Raspi-sump, a sump pump monitoring system.
# Al Audet
# https://www.linuxnorth.org/raspi-sump/
#
# All configuration changes should be done in raspisump.conf
# MIT License -- https://www.linuxnorth.org/raspi-sump/license.html

import configparser
from hcsr04sensor import sensor
from raspisump import log, alerts, heartbeat

config = configparser.RawConfigParser()
config.read("/home/pi/raspi-sump/raspisump.conf")

configs = {
    "critical_water_level": config.getint("pit", "critical_water_level"),
    "pit_depth": config.getint("pit", "pit_depth"),
    "temperature": config.getint("pit", "temperature"),
    "trig_pin": config.getint("gpio_pins", "trig_pin"),
    "echo_pin": config.getint("gpio_pins", "echo_pin"),
    "unit": config.get("pit", "unit"),
}

# If item in raspisump.conf add to configs dict. If not provide defaults.
try:
    configs["alert_when"] = config.get("pit", "alert_when")
except configparser.NoOptionError:
    configs["alert_when"] = "high"

try:
    configs["heartbeat"] = config.getint("email", "heartbeat")
except configparser.NoOptionError:
    configs["heartbeat"] = 0


def initiate_heartbeat():
    """Initiate the heartbeat email process if needed"""
    if configs["heartbeat"] == 1:
        heartbeat.determine_if_heartbeat()
    else:
        pass


def water_reading():
    """Initiate a water level reading."""
    pit_depth = configs["pit_depth"]
    trig_pin = configs["trig_pin"]
    echo_pin = configs["echo_pin"]
    temperature = configs["temperature"]
    unit = configs["unit"]

    value = sensor.Measurement(trig_pin, echo_pin, temperature, unit)

    try:
        raw_distance = value.raw_distance(sample_wait=0.3)
    except SystemError:
        log.log_event("error_log",
                      "ERROR - Signal not received. Possible cable or sensor problem.")
        exit(0)

    return round(value.depth(raw_distance, pit_depth), 1)


def water_depth():
    """Determine the depth of the water, log result and generate alert
    if needed.
    """

    critical_water_level = configs["critical_water_level"]
    water_depth = water_reading()
    if water_depth < 0.0:
        water_depth = 0.0
        log.log_reading(water_depth)
        log.log_event("error_log",
                      "ERROR - Negative reading adjusted to 0.0. Possible degrading sensor.")
    log.log_reading(water_depth)

    if water_depth > critical_water_level and configs["alert_when"] == "high":
        alerts.determine_if_alert(water_depth)
    elif water_depth < critical_water_level and configs["alert_when"] == "low":
        alerts.determine_if_alert(water_depth)
    else:
        pass

    initiate_heartbeat()



'''Log waterlevel readings, restarts and alerts.'''

# Raspi-sump, a sump pump monitoring system.
# Al Audet
# https://www.linuxnorth.org/raspi-sump/
#
# All configuration changes should be done in raspisump.conf
# MIT License -- https://www.linuxnorth.org/raspi-sump/license.html

import time


def log_event(logfile, notification):
    _logfile = "/home/pi/raspi-sump/logs/{}".format(logfile)
    with open(_logfile, 'a') as f:
        f.write(time.strftime("%Y-%m-%d %H:%M:%S,")),
        f.write(notification + "\n"),


def log_reading(water_depth):
    '''Log time and water depth reading.'''
    filename = "/home/pi/raspi-sump/csv/waterlevel-{}.csv".format(
        time.strftime("%Y%m%d"))
    with open(filename, 'a') as f:
        f.write(time.strftime("%H:%M:%S,")),
        f.write(str(water_depth) + "\n"),




from setuptools import setup
import os

version = "1.6.1"

homedir = "/home/pi/raspi-sump/"

if os.path.isfile(homedir + "raspisump.conf"):
    cmd = "cp -u " + homedir + "raspisump.conf " + homedir + "raspisump.conf.save"
    os.system(cmd)

raspi_sump_files = [
    "bin/rsump.py",
    "bin/rsumpchart.py",
    "bin/rsumpmonitor.py",
    "bin/rsumpwebchart.py",
    "bin/emailtest",
]

add_files = [
    (homedir + "sample_config", ["conf/raspisump.conf"]),
    (homedir + "sample_config", ["conf/lighttpd.conf"]),
    (homedir + "csv", ["conf/csv/README.md"]),
    (homedir + "logs", ["conf/logs/README.md"]),
    (homedir + "charts", ["conf/charts/README.md"]),
    (homedir + "docs", ["docs/README.md"]),
    (homedir + "docs", ["docs/install.md"]),
    (homedir + "cron", ["cron/README.md"]),
    (homedir + "cron", ["cron/picrontab"]),
    (homedir + "web", ["conf/web/index.html"]),
    (homedir + "web/images", ["conf/web/images/logo.png"]),
    (homedir + "web/css", ["conf/web/css/index.html"]),
    (homedir + "web/css", ["conf/web/css/raspi.css"]),
    (homedir, ["VERSION"])
]

setup(
    name="raspisump",
    version=version,
    description="A sump pit monitoring system for Raspberry Pi",
    long_description_content_type="text/markdown",
    long_description=open("./README.md", "r").read(),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: End Users/Desktop",
        "Natural Language :: English",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.9",
        "Topic :: Home Automation",
        "License :: OSI Approved :: MIT License",
    ],
    author="Al Audet",
    author_email="alaudet@linuxnorth.org",
    url="https://www.linuxnorth.org/raspi-sump/",
    download_url="https://github.com/alaudet/raspi-sump/releases",
    license="MIT License",
    packages=["raspisump"],
    scripts=raspi_sump_files,
    data_files=add_files,
    install_requires=["hcsr04sensor"],
)

if os.path.isdir(homedir):
    cmd = "chown -R pi " + homedir
    os.system(cmd)
    cmd = "chmod 600 " + homedir + "raspisump.conf"
    os.system(cmd)
    cmd = "chmod 600 " + homedir + "sample_config/raspisump.conf"
    os.system(cmd)
